# -*- coding: utf-8 -*-
# @Time    : 2024/9/12 10:32
# @Author  : Jieay
# @File    : drf_spectacular_external.py
from rest_framework import serializers
from drf_spectacular.utils import OpenApiResponse, OpenApiExample, OpenApiTypes
from drf_spectacular.extensions import OpenApiAuthenticationExtension
import logging

logger = logging.getLogger(__name__)


class CustomDrfSpectacularAuthenticationExtension(OpenApiAuthenticationExtension):
    """
    为自定义认证类创建 OpenApiAuthenticationExtension ，此类需要在初始化注册
    """
    target_class = 'utils.frame.model_view_set.CustomBaseSessionAuthentication'  # 自定义认证类指定的完整路径
    name = 'CustomBaseSessionAuthentication'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_security_definition(self, auto_schema):
        return {
            'type': 'apiKey',
            'in': 'header',
            'name': 'Cookie',
            'description': 'Cookie: sessionid=abc123xyz456',
        }


class DrfSResponseBaseSerializer(serializers.Serializer):
    """接口文档返回的序列化器基础类"""
    status = serializers.BooleanField(help_text='状态', default=True)
    code = serializers.IntegerField(help_text='状态码', default=200)
    msg = serializers.CharField(help_text='日志信息', default='')

    def __init__(self, *args, **kwargs):
        # 获取动态字段配置
        dynamic_fields = kwargs.pop('dynamic_fields', None)
        status = kwargs.pop('status', None)
        code = kwargs.pop('code', None)
        msg = kwargs.pop('msg', None)
        super().__init__(*args, **kwargs)

        if dynamic_fields:
            for field_name, field_type in dynamic_fields.items():
                # 动态添加字段
                self.fields[field_name] = field_type

        if status is not None:
            self.fields['status'].default = status

        if code is not None:
            self.fields['code'].default = code

        if msg is not None:
            if isinstance(msg, str):
                self.fields['msg'] = serializers.CharField(help_text='日志信息', default=msg)
            elif isinstance(msg, dict):
                self.fields['msg'] = serializers.DictField(help_text='日志信息', default=msg)
            elif isinstance(msg, (list, tuple)):
                self.fields['msg'] = serializers.ListField(help_text='日志信息', default=msg)


class DrfSResponseSuccessSerializer(DrfSResponseBaseSerializer):
    """接口文档返回的序列化器成功类"""
    status = serializers.BooleanField(help_text='状态', default=True)
    code = serializers.IntegerField(help_text='状态码', default=200)
    msg = serializers.CharField(help_text='日志信息', default='Queried successfully.')
    data = serializers.DictField(help_text='数据', default={})


class DrfSResponseFailedSerializer(DrfSResponseBaseSerializer):
    """接口文档返回的序列化器失败类"""
    status = serializers.BooleanField(help_text='状态', default=False)
    code = serializers.IntegerField(help_text='状态码', default=400)
    msg = serializers.CharField(help_text='日志信息', default='Queried failed.')
    data = serializers.DictField(help_text='数据', default={})


class DrfSResponseQueriedExceptionSerializer(DrfSResponseBaseSerializer):
    """接口文档返回的序列化器异常类"""
    status = serializers.BooleanField(help_text='状态', default=False)
    code = serializers.IntegerField(help_text='状态码', default=507)
    msg = serializers.DictField(help_text='日志信息', default={})
    data = serializers.DictField(help_text='数据', default={})


class SerializerDrfSpectacularResponse:
    """序列化器接口文档返回构造"""
    def custom_response(self, serializer_name, description='', is_dynamic=False, **kwargs):
        """
        序列化器自定义构造返回数据
        Args:
            serializer_name: `serializers.Serializer` 序列化类
            description: `str` 显示名称
            is_dynamic: `bool` 是否动态新增 data 字段
            **kwargs:

        Returns: `OpenApiResponse`
        """
        _fields = {}
        status = kwargs.get('status', None)
        if status is not None:
            _fields['status'] = status
        code = kwargs.get('code', None)
        if code is not None:
            _fields['code'] = code
        msg = kwargs.get('msg', None)
        if msg is not None:
            _fields['msg'] = msg

        serializer_many = kwargs.get('serializer_many', False)

        if is_dynamic is True:
            if serializer_many is True:
                data = serializer_name(many=True)
            else:
                data = serializer_name()

            _fields['dynamic_fields'] = {
                'data': data,
            }
            # 重新构造序列化类，避免接口文档 Schema 类名称重复
            new_class_name = f'{serializer_name.__name__}Schema'  # 构造类名称
            new_serializer_class = type(new_class_name, (DrfSResponseBaseSerializer,), {})  # 构造新类
            _serializer_class = new_serializer_class(**_fields)
        else:
            _serializer_class = serializer_name(**_fields)

        return OpenApiResponse(
            response=_serializer_class,
            description=description
        )

    def api_response(self, data: list):
        """
        序列化器接口文档构造返回数据
        Args:
            data: `list` [{}]
                    'is_dynamic': `bool` 是否启用 data 字段, True
                    'code': `int` 状态码, 200
                    'status': `bool` 状态, True
                    'msg': `str|list|dict` 日志信息, 'Queried successfully.'
                    'serializer_name': `serializers.Serializer` 序列化类, TokenDataSerializer
                    'description': `str` 显示名称, '查询成功'

        Returns: `dict`
        """
        res_data = {
            507: self.custom_response(
                serializer_name=DrfSResponseQueriedExceptionSerializer,
                description='请求异常',
            )
        }
        for i in data:
            code = i.get('code', 0)
            res_data[code] = self.custom_response(**i)
        return res_data


class CustomDrfSpectacularResponse:
    """自定义接口文档返回构造"""

    def response_data(self, code=0, msg='', data=None, status=False):
        """
        返回格式化数据
        Args:
            code: `int` 状态码
            msg: 日志
            data: 数据
            status: `bool` 状态

        Returns: `obj`
        """
        return {
            "status": status,
            "code": code,
            "msg": msg,
            "data": data
        }

    def success_response(self, code=200, msg='Queried successfully.', data=None, status=True):
        """
        接口文档示例返回成功对象类型
        Args:
            code: `int` 状态码
            msg: 日志
            data: 数据
            status: `bool` 状态

        Returns: `obj`
        """
        return OpenApiResponse(
            response=OpenApiTypes.OBJECT,
            examples=[
                OpenApiExample(
                    'Success Example',
                    description="成功",
                    value=self.response_data(code, msg, data, status)
                )
            ]
        )

    def failed_response(self, code=400, msg='Queried failed.', data=None, status=False):
        """
        接口文档示例返回失败对象类型
        Args:
            code: `int` 状态码
            msg: 日志
            data: 数据
            status: `bool` 状态

        Returns: `obj`
        """
        return OpenApiResponse(
            response=OpenApiTypes.OBJECT,
            examples=[
                OpenApiExample(
                    'Failed Example',
                    description="失败",
                    value=self.response_data(code, msg, data, status)
                )
            ]
        )

    def custom_response(self, name, code, msg, data=None, status=False, description=''):
        """
        接口文档示例返回自定义对象类型
        Args:
            name: Examples名称
            code: `int` 状态码
            msg: 日志
            data: 数据
            status: `bool` 状态
            description: Example Description

        Returns: `obj`
        """
        if not description:
            description = name
        return OpenApiResponse(
            response=OpenApiTypes.OBJECT,
            examples=[
                OpenApiExample(
                    name=name,
                    description=description,
                    value=self.response_data(code, msg, data, status)
                )
            ]
        )

    def api_response(self, data: dict):
        """
        接口文档生成示例返回对象 Responses
        Args:
            data: `dict` 状态码为键，值是 OpenApiResponse 对象

        Returns: `obj`
        """
        res_data = {
            507: self.custom_response(
                name='QueriedException',
                code=507,
                msg={},
                data={},
                description='请求异常'
            )
        }
        res_data.update(data)
        return res_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/20 4:37 下午
# @Author  : Jieay
# @File    : members_serializer.py

from rest_framework import serializers
from drf_spectacular.utils import extend_schema_field
from app.models.user.members import Members


class MembersModelSerializer(serializers.ModelSerializer):
    department_info = serializers.SerializerMethodField(help_text="部门列表详细信息字段")

    # 生成接口文档时，使用 @extend_schema_field 手动指定字段的类型
    @extend_schema_field({
        'type': 'object',
        'properties': {
            'id': {'type': 'integer'},
            'name': {'type': 'string'},
        }
    })
    def get_department_info(self, obj):
        return obj.department_info

    class Meta:
        model = Members
        fields = '__all__'


class ListMembersModelSerializer(MembersModelSerializer):
    """列表接口序列化"""

    class Meta:
        model = Members
        fields = [
            'id', 'username', 'name', 'email', 'unit_name', 'dept_name', 'employee_code',
            'employee_name', 'oaid', 'post_name', 'telephone', 'department_info'
        ]


class CreateMembersModelSerializer(MembersModelSerializer):
    """新增接口序列化"""

    def __init__(self, *args, **kwargs):
        super(CreateMembersModelSerializer, self).__init__(*args, **kwargs)
        self.fields['name'].required = True
        self.fields['email'].required = True
        self.fields['oaid'].required = True

    class Meta:
        model = Members
        exclude = ('password', 'access_key', 'secret_key', 'groups', 'user_permissions')


class UpdateMembersModelSerializer(CreateMembersModelSerializer):
    """更新接口序列化"""
    pass


class PartialUpdateMembersModelSerializer(MembersModelSerializer):
    """部分更新接口序列化"""

    class Meta:
        model = Members
        exclude = ('password', 'access_key', 'secret_key', 'groups', 'user_permissions')


class RetrieveMembersModelSerializer(MembersModelSerializer):
    """检索接口序列化"""

    class Meta:
        model = Members
        exclude = ('password', 'access_key', 'secret_key', 'groups', 'user_permissions')


class CreateDrfSRequestMembersSerializer(serializers.Serializer):
    """创建用户，接口文档请求Body参数"""
    name = serializers.CharField(required=True, help_text='用户')
    username = serializers.CharField(required=True, help_text='用户名')
    password = serializers.CharField(required=True, help_text='密码')
    email = serializers.EmailField(required=True, help_text='邮箱地址')
    oaid = serializers.CharField(required=True, help_text='OAID')
    unit_name = serializers.CharField(required=False, help_text='公司名称')
    dept_name = serializers.CharField(required=False, help_text='部门名称')
    employee_code = serializers.CharField(required=False, help_text='员工编号')
    employee_name = serializers.CharField(required=False, help_text='员工姓名')
    post_name = serializers.CharField(required=False, help_text='职位')


class DestroyDrfSRequestMembersSerializer(serializers.Serializer):
    """删除用户，接口文档请求Body参数"""
    username = serializers.CharField(required=True, help_text='用户名')


class DestroyDrfSResponseMembersSerializer(serializers.Serializer):
    """删除用户，接口文档返回data数据"""
    id = serializers.IntegerField(help_text='ID')


class GetDetailDrfSResponseMembersSerializer(serializers.ModelSerializer):
    """自定义用户详情，接口文档返回data数据"""

    class Meta:
        model = Members
        fields = [
            'id', 'username', 'name', 'email', 'unit_name', 'dept_name', 'employee_code',
            'employee_name', 'oaid', 'post_name', 'telephone', 'is_active', 'is_superuser', 'last_login'
        ]


class GetDrfSRequestMembersSerializer(serializers.Serializer):
    """获取Token，接口文档请求Body参数"""
    accesskey = serializers.CharField(required=True, help_text='AK')
    secretkey = serializers.CharField(required=True, help_text='SK')


class GetDrfSResponseMembersSerializer(serializers.Serializer):
    """获取Token，接口文档返回data数据"""
    token = serializers.CharField(help_text='Token')



# -*- coding: utf-8 -*-
# @Time    : 2024/8/16 15:51
# @Author  : Jieay
# @File    : settings_model.py

from django.db import models


class SettingConfigsModel(models.Model):
    name = models.CharField(max_length=128, unique=True, verbose_name='配置名称(KEY)')
    content = models.TextField(default='{}', blank=True, null=True, verbose_name="配置内容(VALUE)格式JSON")
    ctime = models.DateTimeField(auto_now_add=True, verbose_name='创建时间')
    mtime = models.DateTimeField(auto_now=True, verbose_name='修改时间')
    comments = models.TextField(blank=True, null=True, verbose_name='备注', help_text='备注信息。')

    def __str__(self):
        return self.name

    class Meta:
        db_table = "app_common_setting_configs"
        verbose_name_plural = "(SC)公共配置信息"



from django.apps import AppConfig as AppCfg


class AppConfig(AppCfg):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'app'

    # ready() 是 Django 应用程序启动时执行初始化逻辑的地方。
    # 常见用途包括信号注册、扩展注册、模块导入、以及其他初始化任务。
    # 不要直接在 ready() 中进行数据库操作，避免循环导入问题。
    def ready(self):
        # 注册自定义扩展
        from utils.frame.drf_spectacular_external import CustomDrfSpectacularAuthenticationExtension
        CustomDrfSpectacularAuthenticationExtension('utils.frame.model_view_set.CustomBaseSessionAuthentication')



# -*- coding: utf-8 -*-
# @Time    : 2024/8/16 15:53
# @Author  : Jieay
# @File    : settings_docs.py
from utils.comm.commapi import Choices

"""
自定义公共配置信息，在管理台操作配置

使用方式：
from utils.comm.settings_api import SettingConfig
sc = SettingConfig()
demo_test = sc.get_value(name='DEMO_TEST')
"""


class ConstSettingConfigData(Choices):
    """
    自定义公共配置信息，注册配置 KEY 和说明
    配置内容存储在数据库中，可以实现热更新配置。使用场景：分布式微服务、动态环境变量等
    """

    SC_000001 = ('DEMO_TEST', '数据库配置环境变量，动态生效。')



# -*- coding: utf-8 -*-
# @Time    : 2024/8/16 15:52
# @Author  : Jieay
# @File    : settings_api.py
import json
from utils.comm.commapi import check_json_format, str_to_int, get_function_name
from app.models.common.settings_model import SettingConfigsModel
from utils.comm.settings_docs import ConstSettingConfigData
import logging
logger = logging.getLogger(__name__)


class SettingConfig(object):
    """数据库自定义系统配置管理API"""

    # 获取自定义公共配置注册数据
    setting_config_data = ConstSettingConfigData()

    def docs(self):
        """
        获取所有注册配置信息
        Returns: `list` [('配置KEY', '配置说明')]
        """
        return ConstSettingConfigData.choices()

    def sc_keys(self):
        """
        获取所以注册配置 KEY 列表
        Returns: `list` ['KEY1', 'KEY2']
        """
        return ConstSettingConfigData.keys()

    def sc_values(self):
        """
        获取所以注册配置 KEY 说明列表
        Returns: `list` ['说明1', '说明2']
        """
        return ConstSettingConfigData.values()

    def sc_obj(self, **kwargs):
        """
        获取数据库实例
        Args:
            **kwargs: 数据库字段和值

        Returns: `obj` model
        """
        try:
            db_obj = SettingConfigsModel.objects.filter(**kwargs)
        except Exception as e:
            logger.warning(e)
            db_obj = None
        return db_obj

    def sc_obj_all(self):
        """
        获取数据库中所有配置
        Returns: `obj` model
        """
        if self.sc_obj() is None:
            return None
        return self.sc_obj().all()

    def dict(self):
        """
        获取公共配置键值对字典，来自数据库数据
        Returns: `dict` {'KEY1': 'VALUE1', 'KEY2': 'VALUE2'}
        """
        if self.sc_obj_all() is None:
            return {}
        data = dict(self.sc_obj_all().values_list('name', 'content'))
        return data

    def keys(self):
        """
         获取公共配置所有键，来自数据库数据
        Returns: `list` ['KEY1', 'KEY2']
        """
        return list(self.dict().keys())

    def check_key(self, name):
        """
        检查数据库中公共配置 KEY 是否存在， 不存在则新增
        Args:
            name: KEY

        Returns: `bool`
        """
        check_obj = self.sc_obj(name=name)
        if check_obj is None:
            return False
        comments = ConstSettingConfigData.get(key=name)
        if comments:
            if not check_obj.exists():
                SettingConfigsModel.objects.update_or_create(name=name, comments=comments)
                return False
            else:
                check_obj.update(comments=comments)
        return True

    def get_value(self, name, default=None, s_t_i=False):
        """
        获取指定 KEY 的值
        Args:
            name: KEY
            default: 默认值，在数据库中未查询到值，则返回默认值
            s_t_i: 是否字符串转换数字

        Returns: value
        """
        __c_f_name = get_function_name(c_name=self.__class__.__name__)
        try:
            self.check_key(name=name)
            data = self.dict().get(name, None)
            if check_json_format(data):
                data = json.loads(data)
            if not data and default is not None:
                data = default
            if s_t_i is True and data:
                data = str_to_int(data)
            return data
        except Exception as e:
            logger.warning('{} 获取公共自定义配置值异常，name: {}'.format(__c_f_name, name))
            logger.warning('{} {}'.format(__c_f_name, e))
            return None

    def check_keys(self):
        """检查自定义公共配置，用于初始化注册 key 更新到数据库"""
        __c_f_name = get_function_name(c_name=self.__class__.__name__)
        try:
            logger.info('自定义公共配置，开始检查')
            sc_keys = self.sc_keys()
            for key in sc_keys:
                self.check_key(name=key)
            logger.info('自定义公共配置，检查完成')
        except Exception as e:
            logger.warning('{} 自定义公共配置，检查出错：{}'.format(__c_f_name, e))
            return None

    def update_key_content(self, name, content):
        """
        更新公共配置指定key内容
        Args:
            name: KEY
            content: VALUE

        Returns: `bool`
        """
        self.check_key(name=name)
        check_obj = self.sc_obj(name=name)
        if check_obj.exists():
            if not isinstance(content, str):
                content = json.dumps(content)
            check_obj.update(content=content)
            return True
        return False




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-09-17 08:58
# @Author  : Jieay
# @File    : commapi.py

import os
import re
import sys
import json
import copy
import time
import base64
import string
import random
import uuid
import fcntl
import inspect
import datetime
import functools
import subprocess


from rest_framework.exceptions import APIException


def check_legal_python():
    """检查合法python版本, 3.6 <= python < 3.9"""
    version_major = sys.version_info.major
    version_minor = sys.version_info.minor
    if version_major > 2 and 6 <= version_minor < 9:
        return True
    return False


def func_execution_time(func):
    """
    方法执行时间 装饰器
    :param func:
    :return:
    """
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        start = time.perf_counter()
        res = func(*args, **kwargs)
        end = time.perf_counter()
        print('{} took execution time {} ms'.format(func.__name__, (end - start) * 1000))
        return res
    return wrapper


def get_random(mun, ty=None):
    """
    获取随机唯一字符串
    :param mun: 随机字符长度
    :param ty: 头部前缀字符
    :return:
    """
    if ty == 'tyint':
        restr = string.digits
    elif ty == 'tystr':
        restr = string.ascii_letters
    else:
        restr = string.digits + string.ascii_letters

    str_list = []
    for i in range(int(mun)):
        cs = random.choice(restr)
        str_list.append(cs)
    return ''.join(str_list)


def get_unique_id(title=None):
    """
    :param title: unique_id 的头部， 默认无
    :return: str 默认为28位数字
    """
    today_time = datetime.datetime.now()
    date_time = today_time.strftime("%Y%m%d%H%M%S")
    time_stamp = today_time.strftime("%s")
    time_str = str(date_time) + str(time_stamp)
    random_mun = get_random(4, ty='tyint')
    if title:
        str_id = '{0}{1}{2}'.format(str(title), time_str, random_mun)
    else:
        str_id = '{0}{1}'.format(time_str, random_mun)
    return str_id


def str_to_float(data):
    """
    将字符串转化成浮点数
    """
    try:
        data = float(data)
    except:
        data = data
    return data


def datetime_to_str(o):
    """
    封装将 datetime 时间对象转换字符串格式
    :param o: 时间对象：datetime.datetime or datetime.date
    :return:
    """
    if isinstance(o, datetime.datetime):
        return o.strftime('%Y-%m-%d %H:%M:%S')
    elif isinstance(o, datetime.date):
        return o.strftime('%Y-%m-%d')
    else:
        return str(o)


def get_time_stamp(datetime_obj):
    """
    获取时间戳
    :param datetime_obj: 时间对象
    :return:
    """
    if isinstance(datetime_obj, datetime.datetime):
        return int(time.mktime(datetime_obj.timetuple()))
    elif re.match(r'\d+-\d+-\d+ \d+:\d+:\d+', datetime_obj):
        time_strp = time.strptime(datetime_obj, '%Y-%m-%d %H:%M:%S')
        return int(time.mktime(time_strp))
    return


def check_json_format(raw_msg):
    """
    用于判断一个字符串是否符合Json格式
    :param raw_msg: 源字符
    :return: 布尔值
    """
    if raw_msg == 'null':
        return False

    if isinstance(raw_msg, str):
        try:
            if check_legal_python():
                json.loads(raw_msg, encoding='utf-8')
            else:
                json.loads(raw_msg)
        except ValueError:
            return False
        return True
    else:
        return False


def get_serializer_data_to_json(serializer_data):
    data = copy.deepcopy(serializer_data)
    if isinstance(data, dict):
        for k, v in data.items():
            if check_json_format(v):
                serializer_data[k] = json.loads(v)
            if isinstance(v, dict):
                for kk, vv in v.items():
                    if check_json_format(vv):
                        serializer_data[k][kk] = json.loads(vv)

    if isinstance(data, list):
        for nu, me in enumerate(data):
            if isinstance(me, dict):
                for k, v in me.items():
                    if check_json_format(v):
                        serializer_data[nu][k] = json.loads(v)
                    if isinstance(v, dict):
                        for kk, vv in v.items():
                            if check_json_format(vv):
                                serializer_data[nu][k][kk] = json.loads(vv)
    return serializer_data


class ApiLimitException(APIException):
    status_code = 503
    default_detail = 'Task already exists, try again later.'


class ApiFieldCheckException(APIException):
    status_code = 507
    default_detail = 'Field check exception.'


def api_limit_from_db(model, ck_field, key=None):
    """
    通过查询数据库指定字段条件，限制API接口请求
    :param model: 数据库model
    :param ck_field: 获取 request.data 查询的字段名称
    :param key: 自定义字段筛选条件，类型为 dict
    """
    def decorate(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            assert isinstance(key, dict), ApiLimitException(
                detail="@api_limit_from_db() missing required argument: 'key'"
            )

            request_data = args[1].data
            db_key = {}
            if isinstance(ck_field, (list, tuple)):
                ck_field_list = ck_field
            else:
                ck_field_list = [ck_field]
            for i in ck_field_list:
                db_field = request_data.get(i)
                if db_field is not None:
                    db_key[i] = db_field
            assert db_key, ApiLimitException(
                detail="api_limit_from_db() missing required data: 'ck_field'"
            )

            if isinstance(key, dict):
                key.update(db_key)
                obj = model.objects.filter(**key)
                for ok, ov in db_key.items():
                    key.pop(ok)
                if obj:
                    the_object = True
                else:
                    the_object = False
            else:
                the_object = False
            assert not the_object, ApiLimitException(
                detail="@api_limit_from_db() check db task already exists."
            )

            res = func(*args, **kwargs)
            return res
        return wrapper
    return decorate


def check_request_field(data, ck_field):
    """
    检查请求中是否存在指定的字段
    :param data: 请求数据
    :param ck_field: 指定的字段，可以是多个
    """
    if not isinstance(data, dict):
        raise ApiFieldCheckException(detail='The request is not JSON.')
    field_list = []
    if isinstance(ck_field, (list, tuple)):
        field_list.extend([x for x in ck_field])
    else:
        field_list.append(ck_field)

    ck_field_list = copy.deepcopy(field_list)
    for i in ck_field_list:
        ck_data_field = data.get(i, None)
        if ck_data_field is not None:
            field_list.remove(i)
    if field_list:
        msg_filed = ','.join(field_list)
        raise ApiFieldCheckException(detail='The request data {} is not exist.'.format(msg_filed))

    return data


def check_model_field(data, model, **kwargs):
    """
    检查请求的数据，指定字段是否存在数据库中，is_exist: 默认不存在则报异常，否则存在报异常
    :param data: 请求数据
    :param model: 数据库 model
    :param kwargs: ck_field 指定检查的字段，可以是多个
    """
    field_list = []
    exclude_fields = ['ck_field', 'is_exist', 'check_del']
    ck_field = kwargs.get('ck_field', None)
    if isinstance(ck_field, (list, tuple)):
        field_list.extend(ck_field)
    if ck_field:
        field_list.append(ck_field)
    if isinstance(kwargs, dict):
        _ks = list(kwargs.keys())
        for ef in exclude_fields:
            if ef in _ks:
                _ks.remove(ef)
        field_list.extend(_ks)
    check_request_field(data=data, ck_field=field_list)

    ck_dict = {}
    for i in field_list:
        fd = data.get(i, None)
        if fd:
            ck_dict[i] = fd

    is_exist = kwargs.get('is_exist', False)
    check_del = kwargs.get('check_del', None)
    if check_del:
        ck_dict['soft_del'] = False

    ck_obj = model.objects.filter(**ck_dict)
    if is_exist is True:
        if ck_obj.exists():
            msg_filed = ', '.join(ck_dict)
            raise ApiFieldCheckException(detail='The request data {} is exist for model.'.format(msg_filed))
    else:
        if not ck_obj.exists():
            msg_filed = ', '.join(ck_dict)
            raise ApiFieldCheckException(detail='The request data {} is not exist for model.'.format(msg_filed))

    return data


def clean_response_field(data, ck_field):
    """
    删除请求返回值中的字段
    :param data: 返回数据
    :param ck_field: 需要删除的字段
    """
    data_cp = copy.deepcopy(data)

    if not isinstance(data, (dict, list)):
        raise ApiFieldCheckException(detail='The response is not JSON.')
    field_list = []
    if isinstance(ck_field, (list, tuple)):
        field_list.extend([x for x in ck_field])
    else:
        field_list.append(ck_field)

    if isinstance(data, dict):
        for k, v in data_cp.items():
            if k in field_list:
                data.pop(k)
    if isinstance(data, list):
        for m, obj in enumerate(data_cp):
            if isinstance(obj, dict):
                for kk, vv in obj.items():
                    if kk in field_list:
                        data[m].pop(kk)
    return data


def get_model_field_data(model, ck_field, *args, **kwargs):
    """
    获取指定字段的数据，并且可以指定字段排序
    :param model: 数据库 model
    :param ck_field: 自定字段，单个或者一个列表
    :param args: 指定字段，多个字段
    :param kwargs: 排序字段order_by， 正序'xxx'|倒序'-xxx'
    """
    filed_list = []
    if isinstance(ck_field, (list, tuple)):
        filed_list.extend(ck_field)
    else:
        filed_list.append(ck_field)
    for i in args:
        if isinstance(i, (list, tuple)):
            filed_list.extend(i)
        else:
            filed_list.append(i)
    order_by = kwargs.get('order_by')
    if order_by is not None:
        ck_obj = model.objects.all().order_by(order_by).values(*filed_list)
    else:
        ck_obj = model.objects.all().values(*filed_list)
    return list(ck_obj)


def check_dict_fields(data, ck_fields):
    """
    检查数据字典中，查询的字段不存在值则 返回 False
    :param data: 数据字典
    :param ck_fields: 查询字段 列表
    :return: False or True
    """
    if not isinstance(ck_fields, (list, tuple)):
        ck_fields = [ck_fields]

    _info = copy.deepcopy(ck_fields)
    if isinstance(data, dict):
        for i in ck_fields:
            ck_v = data.get(i, None)
            if ck_v is not None:
                _info.remove(i)
    if _info:
        return False
    return True


def str_to_int(data):
    """
    将字符串转化成数字
    :param data: str
    :return: int
    """
    try:
        data_info = int(data)
    except:
        data_info = data
    return data_info


def get_str_to_class(data):
    """
    解析对象字符串，将字符串转化成类名称和包的路径
    :param data: 对象字符串
    :return: (类名称, 包路径)
    """
    # data = "<class 'app.models.asset.envs_model.EnvsModel'>"
    data = data.replace('<', '').replace('>', '').replace("'", "").split(' ')
    if len(data) == 2:
        path_str = None
        m_name = None
        if data[0] == 'class':
            p_k = data[1].split('.')
            path_str = '.'.join(p_k[:-1])
            m_name = p_k[-1]
        return m_name, path_str
    return


def get_model_data_from_id(_id, model):
    """
    通过 ID 获取数据库中该条记录的数据
    :param _id: ID
    :param model: 表对象
    :return: `dict`
    """
    obj = model.objects.filter(id=_id).values()
    return obj[0]


def set_mark_delete(_id, model, **kwargs):
    """
    数据软删除，更新数据的标记记录
    :param _id: ID
    :param model: model
    :param kwargs: soft_del=True
    """
    if not kwargs:
        kwargs['soft_del'] = True
    obj = model.objects.filter(id=_id).update(**kwargs)
    return obj


def set_base_response_data(data=None):
    """
    处理自定义接口返回值的 data 数据中的状态字段
    """
    _data = {}
    if isinstance(data, dict):
        _data = copy.deepcopy(data)
        for i in ['status', 'code', 'msg']:
            if data.get(i, None) is not None:
                _data.pop(i)
    else:
        if data is not None:
            _data = data
    return _data


def check_telephone(telephone):
    """
    检查手机号码的合法性
    :param telephone: 手机号字符
    """
    regx = re.match('^[1-9][1-9][0-9]{9}$', telephone)
    if regx is None:
        return False
    return True


def modify_dict_mapping_dict(data, mapping_dict):
    """
    实现将一个字典的key映射成另外的key
    :param data: 要进行处理的数据字典，旧数据字典
    :param mapping_dict: 映射表，字典类型
    :return: 进过修改key名字的新数据字典
    """
    new_data = {}
    # 判断是否是JSON格式
    if check_json_format(data):
        data = json.loads(data)

    # 判断传入的参数是否为dict类型
    if not isinstance(data, dict):
        return new_data
    if not isinstance(mapping_dict, dict):
        return new_data
    for d_k, d_v in data.items():
        if check_json_format(d_v):
            d_v = json.loads(d_v)
        n_d_v = copy.deepcopy(d_v)
        # if isinstance(d_v, list):
        #     if d_v and isinstance(d_v[0], dict):
        #         n_d_v = d_v[0].get('name')

        if isinstance(n_d_v, (list, dict, tuple)):
            n_d_v = json.dumps(n_d_v)

        new_ks = mapping_dict.get(d_k)
        if isinstance(new_ks, list):
            for kk in new_ks:
                new_data[kk] = n_d_v
        else:
            if new_ks:
                new_data[new_ks] = n_d_v
    return new_data


def remove_fields_not_exists_model(model, dict_data):
    """
    删除不存在model字段中的dict keys
    :param model: 数据库模型
    :param dict_data: 字典对象
    :return:
    """
    field_names = [f.name for f in model._meta.fields]
    temp_dict = copy.copy(dict_data)
    for key in temp_dict.keys():
        if key not in field_names:
            dict_data.pop(key)


def order_time_mg(location_datetime, time_type=None, leadtime=10, timeout=10):
    """
    订单超时或者提前时间管理
    :param location_datetime: 指定时间
    :param time_type: 时间类型，超时或者提前，[ leadtime | timeout ]
    :param leadtime: 提前几分钟， 默认提前10分钟
    :param timeout: 超时几分钟，默认10分钟
    :return: bool
    """
    ck_time = False
    location_stamp = get_time_stamp(location_datetime)

    if time_type == 'timeout':
        # 超时
        timeout_limit_datetime = datetime.datetime.now() - datetime.timedelta(minutes=timeout)
        timeout_limit_stamp = get_time_stamp(timeout_limit_datetime)
        if timeout_limit_stamp > location_stamp:
            ck_time = True
    elif time_type == 'leadtime':
        # 提前
        leadtime_limit_datetime = datetime.datetime.now() + datetime.timedelta(minutes=leadtime)
        leadtime_limit_stamp = get_time_stamp(leadtime_limit_datetime)
        if leadtime_limit_stamp > location_stamp:
            ck_time = True

    return ck_time


def get_function_name(c_name=None):
    """
    动态获取正在运行函数(或方法)名称
    :param c_name: 类名称
    """
    if c_name is not None:
        f_name = '%s.%s' % (c_name, inspect.stack()[1][3])
    else:
        f_name = '%s' % inspect.stack()[1][3]
    return f_name


def func_exe_retry(ExceptionToCheck, tries=4, delay=3, back_off=2, logger=None):
    """
    函数运行重试装饰器
    :param ExceptionToCheck: 异常参数元组（TypeError,ValueError）
    :param tries: 重试次数
    :param delay: 重试延时时间 （秒）
    :param back_off: 重试延时倍数
    :param logger: 日志参数
    """
    def deco_retry(f):
        @functools.wraps(f)
        def f_retry(*args, **kwargs):
            m_tries, m_delay = tries, delay
            while m_tries > 1:
                try:
                    return f(*args, **kwargs)
                except ExceptionToCheck as e:
                    msg = "%s, Retrying in %d seconds..." % (str(e), m_delay)
                    if logger:
                        logger.warning(msg)
                    else:
                        print(msg)
                    time.sleep(m_delay)
                    m_tries -= 1
                    m_delay *= back_off
            return f(*args, **kwargs)
        return f_retry  # true decorator
    return deco_retry


def random_sleep_ms():
    """随机毫秒休眠"""
    sleep_time = random.randint(0, 100) * 0.01
    time.sleep(sleep_time)
    return sleep_time


def filter_attrs(attrs):
    for k, v in attrs.items():
        if k.startswith("__"):
            yield k, v
        else:
            if isinstance(v, tuple):
                yield k, v[0]
                yield f'{k}_NAME', v[1]
            else:
                yield k, v


class ChoicesMeta(type):
    def __new__(cls, name, bases, attrs):
        real_attrs = dict(filter_attrs(attrs))
        cls = super(ChoicesMeta, cls).__new__(cls, name, bases, real_attrs)
        cls._dict = {v[0]: v[1] for k, v in attrs.items() if not k.startswith("__") and isinstance(v, tuple)}
        cls._items = list(cls._dict.items())
        cls._values = list(map(lambda x: x[1], cls._items))
        cls._keys = list(map(lambda x: x[0], cls._items))
        return cls

    def __contains__(self, val):
        return val in self.keys()

    def __getitem__(self, k):
        return self._dict[k]

    def keys(cls):
        return cls._keys

    def values(self):
        return self._values

    def items(self):
        return self._items

    def dict(self):
        return self._dict

    def get(self, key, default=None):
        return self._dict.get(key, default)

    def choices(self):
        return self.items()


class Choices(object, metaclass=ChoicesMeta):
    """ 自定义选择常量构造的用法
        In [18]: class FeedType(Choices):
            ...:     IMAGE = (1, '⽂文字图⽚片')
            ...:     VIDEO = (2, '⽂文字视频')
            ...:     ARTICLE = (3, '⽂文章')
            ...:

        In [19]: FeedType.IMAGE == 1
        Out[19]: True

        In [20]: FeedType.IMAGE_NAME == '文字图片'
        Out[20]: False

        In [21]: FeedType.IMAGE in FeedType
        Out[21]: True

        In [22]: FeedType.keys()
        Out[22]: [1, 2, 3]

        In [23]: FeedType.values()
        Out[23]: ['⽂文字图⽚片', '⽂文字视频', '⽂文章']

        In [24]: FeedType.items()
        Out[24]: [(1, '⽂文字图⽚片'), (2, '⽂文字视频'), (3, '⽂文章')]

        In [25]: FeedType.dict()
        Out[25]: {1: '⽂文字图⽚片', 2: '⽂文字视频', 3: '⽂文章'}

        In [26]: FeedType.choices()
        Out[26]: [(1, '⽂文字图⽚片'), (2, '⽂文字视频'), (3, '⽂文章')]

        In [27]: FeedType.get(FeedType.IMAGE)
        Out[27]: '⽂文字图⽚片'
    """


class ModelChoicesMeta(type):
    """数据库模型选择常量元类"""
    def __new__(cls, name, bases, data):
        cls = super(ModelChoicesMeta, cls).__new__(cls, name, bases, data)
        model_name, fields = data.get('data', [None, []])
        data_dict = []
        if model_name:
            try:
                ck_obj = model_name.objects.all().values_list(*fields)
                if ck_obj:
                    data_dict = list(ck_obj)
            except Exception as e:
                print(e)
        cls._dict = dict(data_dict)
        cls._items = list(cls._dict.items())
        cls._values = cls._dict.values()
        cls._keys = cls._dict.keys()
        return cls

    def __contains__(self, val):
        return val in self.keys()

    def __getitem__(self, k):
        return self._dict[k]

    def keys(cls):
        return cls._keys

    def values(self):
        return self._values

    def items(self):
        return self._items

    def dict(self):
        return self._dict

    def get(self, key, default=None):
        return self._dict.get(key, default)

    def choices(self):
        return self.items()


class ModelChoices(object, metaclass=ModelChoicesMeta):
    """
    使用数据库模型，指定字段，构造自定义选择常量的用法
    In [2]: class OAWorkFlowTypeChoices(ModelChoices):
       ...:
       ...:     data = (OAWorkFlowTypesModel, ('number', 'name'))
       ...:

    In [3]: OAWorkFlowTypeChoices.choices()
    Out[3]: [(1, 'KJ008'), (2, 'KJ021')]

    In [4]: OAWorkFlowTypeChoices.get(1)
    Out[4]: 'KJ008'

    In [5]: OAWorkFlowTypeChoices.get(2)
    Out[5]: 'KJ021'

    In [6]: OAWorkFlowTypeChoices.keys()
    Out[6]: dict_keys([1, 2])

    In [7]: OAWorkFlowTypeChoices.values()
    Out[7]: dict_values(['KJ008', 'KJ021'])

    In [8]: OAWorkFlowTypeChoices.items()
    Out[8]: [(1, 'KJ008'), (2, 'KJ021')]

    In [9]: OAWorkFlowTypeChoices.dict()
    Out[9]: {1: 'KJ008', 2: 'KJ021'}

    """


def set_custom_transaction_lock_data(new_data, old_data):
    """
    构造自定义事务锁数据更新
    :param new_data: `dict` 新数据库数据，数据库字段的对象数据需要反序列化，json.loads
    :param old_data: `dict` 旧的数据
    """
    # 防止数据调用被更新，做深度拷贝
    data = copy.deepcopy(new_data)
    _old_data = copy.deepcopy(old_data)
    # 遍历旧的数据，取出字段和字段值
    for o_k, o_v in _old_data.items():
        new_v = new_data.get(o_k)
        # 判断新的字段值是否为序列化对象
        if check_json_format(new_v):
            new_v = json.loads(new_v)
        # 判断旧的字段值是否为序列化对象
        if check_json_format(o_v):
            o_v = json.loads(o_v)

        # 新字段值是否为空
        if new_v:
            # 将新字段值，做深度拷贝
            _new_v = copy.deepcopy(new_v)
            if isinstance(new_v, list) and isinstance(o_v, list):
                new_vv = _new_v
                for ov in o_v:
                    if ov not in new_vv:
                        new_vv.append(ov)
                data[o_k] = json.dumps(new_vv)

            elif isinstance(new_v, dict) and isinstance(o_v, dict):
                new_vv = _new_v.update(o_v)
                data[o_k] = json.dumps(new_vv)

            elif new_v and o_v:
                if isinstance(o_v, (list, dict)):
                    o_v = json.dumps(o_v)
                data[o_k] = o_v
            else:
                pass
        else:
            if isinstance(o_v, (list, dict)):
                o_v = json.dumps(o_v)
            data[o_k] = o_v
    return data


def get_command_exec_result(command_str):
    """
    执行shell命令，且返回执行结果
    :param command_str: shell 命令字符串
    :return: (0, stdout)
    """
    return subprocess.getstatusoutput(command_str)


def check_and_mkdir_file_path_dir(file_path, is_dir=False):
    """
    检查传入文件的目录是否存在， 没有则创建
    :param file_path: 文件绝对路径
    :param is_dir: 传入路径是否只包含目录路径
    :return:
    """
    if not is_dir:
        file_dir_name = os.path.dirname(file_path)
    else:
        file_dir_name = file_path
    try:
        if not os.path.exists(file_dir_name):
            os.makedirs(file_dir_name)
        return True
    except Exception as e:
        return False


def write_text_file_add_lock(data, file_path, write_type="w"):
    """
    编写文本文件(加等待锁， 防止写入冲突)
    :param data: str
    :param file_path: 写入文件绝对路径, str
    :param write_type: 写类型, str, w或a, 即覆盖或追加
    :return:
    """
    try:
        check_and_mkdir_file_path_dir(file_path=file_path)
        with open(file_path, write_type) as f:
            fcntl.flock(f, fcntl.LOCK_EX)
            f.write(data)
            fcntl.flock(f, fcntl.LOCK_UN)
        return True
    except Exception as e:
        return False


def get_a_uuid():
    """
    获取一个UUID
    :return: str
    """
    uuid_obj = uuid.uuid4()
    uuid_str = str(uuid_obj)
    return uuid_str


def get_cur_host_mac_addr():
    """
    获取当前主机节点MAC地址
    :return:
    """
    addr = hex(uuid.getnode())[2:].upper()
    standard_addr = '0' * (12 - len(addr)) + addr
    standard_mac_addr = '-'.join(standard_addr[i:i + 2] for i in range(0, len(standard_addr), 2))
    return standard_mac_addr


def is_ip(ip_str):
    """检查字符串是否是IP"""
    ip_str = str(ip_str)
    p = re.compile(r'^((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$')
    if p.match(ip_str):
        return True
    else:
        return False


def datetime_str_to_str(dt_str):
    """
    将指定时间字符串转化标准格式
    Args:
        dt_str: 2024-08-18T19:59:00.194116

    Returns: `str` 2024-08-18 19:59:00
    """
    datetime_obj = datetime.datetime.strptime(dt_str, '%Y-%m-%dT%H:%M:%S.%f')
    return datetime_obj.strftime('%Y-%m-%d %H:%M:%S')


def bytes_to_str(data):
    """
    将 bytes 类型转化成字符串
    Args:
        data: 数据

    Returns: `str`
    """
    if isinstance(data, bytes):
        return data.decode(encoding='utf-8')
    return data


def str_to_bytes(data):
    """
    将字符串转化成 bytes 类型
    Args:
        data: 数据

    Returns: `bytes`
    """
    if isinstance(data, str):
        return data.encode(encoding='utf-8')
    return data


def get_base64_encoded(data, out_str=True):
    """
    获取 Base64 编码
    Args:
        data: `str`|`bytes` 需要编码字符
        out_str: `bool` 是否输出字符串

    Returns: `str`|`bytes`
    """
    if isinstance(data, bytes):
        _data = data
    else:
        # 将字符串编码为字节
        _data = data.encode('utf-8')
    # 编码为 Base64
    encoded_data = base64.b64encode(_data)
    if out_str is True:
        return encoded_data.decode('utf-8')
    return encoded_data


def get_base64_decode(encoded_data, out_str=True):
    """
    获取 Base64 解码
    Args:
        encoded_data: `str`|`bytes` Base64编码字符
        out_str: `bool` 是否输出字符串

    Returns: `str`|`bytes`
    """
    if isinstance(encoded_data, bytes):
        data = encoded_data
    else:
        # 将字符串编码为字节
        data = encoded_data.encode('utf-8')
    # 解码 Base64 数据
    decoded_data = base64.b64decode(data)
    if out_str is True:
        return decoded_data.decode('utf-8')
    return decoded_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/20 4:25 下午
# @Author  : Jieay
# @File    : __init__.py.py
from django.conf import settings
from app.admin.user.members_admin import *
from app.admin.common.settings_admin import *

admin.site.site_header = settings.PROJECT_NAME
admin.site.site_title = settings.PROJECT_NAME



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import redis
from django.conf import settings


class MyRedis(object):

    def __init__(self, **redis_info):
        """
        :param redis_info: host, port, db, password
        """
        self.host = redis_info.get('host', settings.REDIS_HOST)
        self.port = redis_info.get('port', settings.REDIS_PORT)
        self.db = redis_info.get('db', settings.REDIS_BASE_DB)
        self.passwd = redis_info.get('password', settings.REDIS_PASSWORD)
        redis_info = {
            "host": self.host,
            "port": self.port,
            "db": self.db
        }
        if self.passwd:
            redis_info['password'] = self.passwd
        redis_pool_info = redis.ConnectionPool(**redis_info)
        self.__r = redis.Redis(connection_pool=redis_pool_info)

    def conn(self):
        """redis实例化连接"""
        return self.__r

    def set(self, key, value, timeout=None, nx=False):
        """
        添加键值对
        Args:
            key: 键
            value: 值
            timeout: 过期时间 (s)
            nx: True时存在不会创建

        Returns:
        """
        self.__r.set(name=key, value=value, nx=nx)
        if timeout:
            self.__r.expire(key, timeout)

    def get(self, key):
        """
        通过 KEY 获取 VALUE
        Args:
            key: 键

        Returns: `bytes` 值
        """
        return self.__r.get(key)

    def exists(self, key):
        """
        查询 KEY 是否存在
        Args:
            key: 键

        Returns: `int` 1 or 0

        """
        return self.__r.exists(key)

    def l_push(self, key, value):
        """
        Redis 列表(List), 指定列表名称 key , 从左 push 值
        Args:
            key: 键，列表名称
            value: 列表元素

        Returns: `int` 返回列表的长度
        """
        return self.__r.lpush(key, value)

    def r_push(self, key, value):
        """
        Redis 列表(List), 指定列表名称 key , 从右 push 值
        Args:
            key: 键，列表名称
            value: 列表元素

        Returns: `int` 返回列表的长度
        """
        return self.__r.rpush(key, value)

    def l_pop(self, key):
        """
        Redis 列表(List), 指定列表名称 key , 从左 Pop 值, 拿出即删除
        Args:
            key: 键，列表名称

        Returns: `bytes` 值
        """
        return self.__r.lpop(key)

    def r_pop(self, key):
        """
        Redis 列表(List), 指定列表名称 key , 从右 Pop 值, 拿出即删除
        Args:
            key: 键，列表名称

        Returns: `bytes` 值
        """
        return self.__r.rpop(key)

    def l_len(self, key):
        """
        Redis 列表(List), 获取列表长度
        Args:
            key: 键，列表名称

        Returns: `int` 返回列表的长度
        """
        return self.__r.llen(key)

    def l_range(self, key, start, end):
        """
        Redis 列表(List), 获取列表制定位置的元素
        Args:
            key: 键，列表名称
            start: 开始索引
            end: 结束索引

        Returns: `list` [bytes]
        """
        return self.__r.lrange(key, start, end)

    def h_set(self, key, field, value):
        """
        Redis 哈希(Hash), 用于为哈希表中的字段赋值
        Args:
            key: 键，哈希表名称
            field: 哈希表中字段
            value: 哈希表中字段的值

        Returns: `int` 1(新增) or 0(修改)
        """
        return self.__r.hset(key, field, value)

    def h_m_set(self, key, mapping):
        """
        Redis 哈希(Hash), 用于为哈希表中的多对字段赋值
        Args:
            key: 键，哈希表名称
            mapping: `dict` {str: str}

        Returns: `bool`
        """
        return self.__r.hmset(key, mapping)

    def h_get(self, key, field):
        """
        Redis 哈希(Hash), 获取哈希表中的字段的值
        Args:
            key: 键，哈希表名称
            field: 哈希表中字段

        Returns: `bytes`
        """
        return self.__r.hget(key, field)

    def h_exists(self, key, field):
        """
        Redis 哈希(Hash), 判断哈希表中的字段是否存在
        Args:
            key: 键，哈希表名称
            field: 哈希表中字段

        Returns: `bool`
        """
        return self.__r.hexists(key, field)

    def h_keys(self, key):
        """
        Redis 哈希(Hash), 获取哈希表中的所有字段
        Args:
            key: 键，哈希表名称

        Returns: `list` [bytes]
        """
        return self.__r.hkeys(key)

    def h_vals(self, key):
        """
        Redis 哈希(Hash), 获取哈希表中的所有字段的值
        Args:
            key: 键，哈希表名称

        Returns: `list` [bytes]
        """
        return self.__r.hvals(key)

    def h_get_all(self, key):
        """
        Redis 哈希(Hash), 获取哈希表中的所有字段和值
        Args:
            key: 键，哈希表名称

        Returns: `dict` {bytes: bytes}
        """
        return self.__r.hgetall(key)

    def h_del(self, key, fields):
        """
        Redis 哈希(Hash), 删除哈希表中指定字段
        Args:
            key: 键，哈希表名称
            *fields: 哈希表中的多个字段列表

        Returns: None
        """
        self.__r.hdel(key, *fields)

    def h_len(self, key):
        """
        Redis 哈希(Hash), 获取哈希表中长度
        Args:
            key: 键，哈希表名称

        Returns: `int`
        """
        return self.__r.hlen(key)



"""
URL configuration for frame_server project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/4.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, re_path, include
from django.conf import settings

from app.views.cas_admin import views as cas_views
from app.views.comm.health_view import GetHealthCheckView
from app.views.user.members_view import APIGetTokenView

urlpatterns = [
    path('admin/', admin.site.urls),
    path('gettoken', APIGetTokenView.as_view()),
    path('login/', cas_views.CasLoginView.as_view(), name='cas_ng_login'),
    path('logout/', cas_views.CasLogoutView.as_view(), name='cas_ng_logout'),
    path('health/check', GetHealthCheckView.as_view()),
]

if settings.DEBUG is True:
    from drf_spectacular.views import SpectacularJSONAPIView, SpectacularRedocView, SpectacularSwaggerView
    urlpatterns += [
        path('swagger/json/', SpectacularJSONAPIView.as_view(), name='schema'),
        # Optional UI:
        path('swagger/ui/', SpectacularSwaggerView.as_view(url_name='schema'), name='swagger-ui'),
        path('swagger/redoc/', SpectacularRedocView.as_view(url_name='schema'), name='redoc'),
        # YOUR PATTERNS
    ]

urlpatterns += [
    re_path(r'^api/', include('app.urls')),
    re_path(r'^sub-server/', include('app.sub_urls')),
]


# -*- coding: utf-8 -*-
# @Time    : 2024/8/16 16:13
# @Author  : Jieay
# @File    : settings_admin.py

from django.contrib import admin
from app.models.common.settings_model import SettingConfigsModel


@admin.register(SettingConfigsModel)
class SettingConfigsModelAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'name', 'content', 'comments', 'ctime', 'mtime'
    )
    list_display_links = ['id', 'name']
    list_per_page = 15
    search_fields = ['id', 'name', 'content']
    list_editable = ('content',)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 18:29
# @Author  : Jieay
# @File    : members_view.py
from rest_framework import status
from rest_framework.authentication import get_authorization_header
from rest_framework.authtoken.models import Token
from django_filters import rest_framework as filters
from rest_framework.filters import OrderingFilter

from utils.frame.model_view_set import CustomBaseViewSet, CustomBaseJsonResponse
from utils.frame.api_view_external import BaseAdminApiView, BaseOpenApiView, BaseResponse
from utils.frame.drf_spectacular_external import CustomDrfSpectacularResponse
from app.models.user.members import Members
from app.serializers.user.members_serializer import MembersModelSerializer, CreateMembersModelSerializer, \
    UpdateMembersModelSerializer, RetrieveMembersModelSerializer, GetDrfSRequestMembersSerializer, \
    PartialUpdateMembersModelSerializer, DestroyDrfSRequestMembersSerializer, DestroyDrfSResponseMembersSerializer, \
    GetDrfSResponseMembersSerializer, \
    ListMembersModelSerializer, GetDetailDrfSResponseMembersSerializer
from utils.comm.commapi import get_serializer_data_to_json, clean_response_field, check_model_field
from utils.user.membersapi import get_aksk_token
from drf_spectacular.utils import extend_schema, OpenApiParameter
from utils.frame.drf_spectacular_external import SerializerDrfSpectacularResponse, DrfSResponseFailedSerializer

import logging

logger = logging.getLogger(__name__)


class MembersView(BaseAdminApiView):
    """成员信息"""

    def get_token(self, request):
        auth = get_authorization_header(request).split()
        if len(auth) == 2:
            key = auth[1].decode()
        else:
            key = None
        token = Token.objects.select_related('user').get(key=key)
        return token

    def get_user_info(self, token):
        data = {}
        v_fields = [
            'id', 'username', 'name', 'email', 'unit_name', 'dept_name', 'employee_code',
            'employee_name', 'oaid', 'post_name', 'telephone', 'is_active', 'is_superuser', 'last_login'
        ]
        ck_obj = Members.objects.filter(id=token.user.id)
        if ck_obj.exists():
            data.update(ck_obj.values(*v_fields)[0])
        return data

    @extend_schema(
        tags=['User'],
        summary='GetV1MembersDetail',
        description='查询用户详情接口',
        methods=['GET'],
        responses=CustomDrfSpectacularResponse().api_response(
            data={
                200: CustomDrfSpectacularResponse().success_response(
                    code=200,
                    data=GetDetailDrfSResponseMembersSerializer().data
                ),
                400: CustomDrfSpectacularResponse().failed_response(
                    code=400,
                    data={}
                )
            }
        ),
    )
    def get(self, request):
        token = self.get_token(request)
        data = {}
        if token is not None:
            data = self.get_user_info(token)
            return BaseResponse(code=200, data=data, status=True, msg='Queried successfully.')
        return BaseResponse(code=400, data=data)


class MembersFilter(filters.FilterSet):
    username = filters.CharFilter(field_name='username', lookup_expr='exact')
    name = filters.CharFilter(field_name='name', lookup_expr='icontains')
    oaid = filters.CharFilter(field_name='oaid', lookup_expr='exact')
    telephone = filters.CharFilter(field_name='telephone', lookup_expr='exact')

    class Meta:
        model = Members
        fields = [
            'id', 'username', 'name', 'oaid', 'telephone'
        ]
        ordering_fields = '__all__'


class MembersViewsSet(CustomBaseViewSet):
    """用户信息"""

    serializer_class = MembersModelSerializer
    queryset = Members.objects.all()
    filter_backends = (filters.DjangoFilterBackend, OrderingFilter)
    filter_class = MembersFilter
    ordering = ['-ctime']
    model_name = Members

    @extend_schema(
        tags=['User'],
        summary='ListV1Users',
        description='查询用户信息列表接口',
        methods=['GET'],
        parameters=[
            OpenApiParameter(name='id', description='用户ID', required=False, type=int),
            OpenApiParameter(name='username', description='用户名', required=False, type=str),
            OpenApiParameter(name='name', description='用户名称', required=False, type=str),
            OpenApiParameter(name='oaid', description='用户OAID', required=False, type=str),
            OpenApiParameter(name='telephone', description='电话', required=False, type=str),
        ],
        responses=ListMembersModelSerializer
    )
    def list(self, request, *args, **kwargs):
        self.serializer_class = ListMembersModelSerializer
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            data = get_serializer_data_to_json(serializer.data)
            data = clean_response_field(data, self.del_fields)
            return self.get_paginated_response(data)

        serializer = self.get_serializer(queryset, many=True)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data)

    @extend_schema(
        tags=['User'],
        summary='CreateV1Users',
        description='新增用户信息接口',
        methods=['POST'],
        request=CreateMembersModelSerializer,
        responses=SerializerDrfSpectacularResponse().api_response(
            data=[
                {
                    'is_dynamic': True,
                    'code': 201,
                    'msg': 'Created successfully.',
                    'serializer_name': CreateMembersModelSerializer,
                    'description': '创建成功'
                }
            ]
        )
    )
    def create(self, request, *args, **kwargs):
        self.serializer_class = CreateMembersModelSerializer
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        logger.info('Create: {}'.format(serializer.data))
        data = get_serializer_data_to_json(serializer.data)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_201_CREATED,
                                      headers=headers,
                                      msg="Created successfully.",
                                      status=True)

    @extend_schema(
        tags=['User'],
        summary='UpdateV1Users',
        description='更新用户信息接口',
        methods=['PUT'],
        request=UpdateMembersModelSerializer,
        responses=SerializerDrfSpectacularResponse().api_response(
            data=[
                {
                    'is_dynamic': True,
                    'code': 200,
                    'msg': 'Updated successfully.',
                    'serializer_name': UpdateMembersModelSerializer,
                    'description': '更新成功'
                }
            ]
        )
    )
    def update(self, request, *args, **kwargs):
        self.serializer_class = UpdateMembersModelSerializer
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            instance._prefetched_objects_cache = {}
        logger.info('Update: {}'.format(serializer.data))
        data = get_serializer_data_to_json(serializer.data)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Updated successfully.",
                                      status=True)

    @extend_schema(
        tags=['User'],
        summary='PartialUpdateV1Users',
        description='更新部分用户信息接口',
        methods=['PATCH'],
        responses=SerializerDrfSpectacularResponse().api_response(
            data=[
                {
                    'is_dynamic': True,
                    'code': 200,
                    'msg': 'Updated successfully.',
                    'serializer_name': PartialUpdateMembersModelSerializer,
                    'description': '更新成功'
                }
            ]
        )
    )
    def partial_update(self, request, *args, **kwargs):
        self.serializer_class = PartialUpdateMembersModelSerializer
        return super(MembersViewsSet, self).partial_update(request, *args, **kwargs)

    @extend_schema(
        tags=['User'],
        summary='RetrieveV1Users',
        description='检索用户信息接口',
        methods=['GET'],
        responses=SerializerDrfSpectacularResponse().api_response(
            data=[
                {
                    'is_dynamic': True,
                    'code': 200,
                    'msg': 'Queried successfully.',
                    'serializer_name': RetrieveMembersModelSerializer,
                    'description': '查询成功'
                }
            ]
        )
    )
    def retrieve(self, request, *args, **kwargs):
        self.serializer_class = RetrieveMembersModelSerializer
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Queried successfully.",
                                      status=True)

    @extend_schema(
        tags=['User'],
        summary='DestroyV1Users',
        description='删除用户信息接口',
        methods=['DELETE'],
        request=DestroyDrfSRequestMembersSerializer,
        responses=SerializerDrfSpectacularResponse().api_response(
            data=[
                {
                    'is_dynamic': True,
                    'code': 200,
                    'msg': 'Deleted successfully.',
                    'serializer_name': DestroyDrfSResponseMembersSerializer,
                    'description': '删除成功'
                }
            ]
        )
    )
    def destroy(self, request, *args, **kwargs):
        check_model_field(request.data, self.model_name, ck_field='username')
        instance = self.get_object()
        _id = instance.id
        self.perform_destroy(instance)
        logger.info('Delete: {}'.format([{"id": _id}]))
        return CustomBaseJsonResponse(data={"id": _id},
                                      code=status.HTTP_200_OK,
                                      msg="Deleted successfully.",
                                      status=True)


class APIGetTokenView(BaseOpenApiView):
    """获取token"""

    @extend_schema(
        tags=['Token'],
        summary='GetToken',
        description='获取Token接口',
        methods=['POST'],
        request=GetDrfSRequestMembersSerializer,
        responses=SerializerDrfSpectacularResponse().api_response(
            data=[
                {
                    'is_dynamic': True,
                    'code': 200,
                    'serializer_name': GetDrfSResponseMembersSerializer,
                    'description': '查询成功'
                },
                {
                    'code': 400,
                    'serializer_name': DrfSResponseFailedSerializer,
                    'description': '查询失败'
                },
            ]
        )
    )
    def post(self, request):
        access_key = request.data.get('accesskey')
        secret_key = request.data.get('secretkey')
        token = get_aksk_token(access_key, secret_key)
        data = {}
        if token is not None:
            data['token'] = token
            return BaseResponse(code=200, data=data, status=True, msg='Queried successfully.')
        return BaseResponse(code=400, data=data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:19
# @Author  : Jieay
# @File    : pagination.py
from rest_framework.pagination import LimitOffsetPagination, PageNumberPagination
from collections import OrderedDict
from rest_framework.response import Response


class CustomBasePagination(LimitOffsetPagination):
    """自定义偏移量分页器基类"""

    def get_paginated_response(self, data):
        status = True
        code = 200
        msg = 'Queried successfully.'
        if not data:
            # status = False  # 查不到，理论来说，只是结果为空，但不是出错。
            code = 404
            msg = "Data not found."

        return Response(OrderedDict([
            ('status', status),
            ('code', code),
            ('msg', msg),
            ('count', self.count),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('data', data),
        ]))

    def get_paginated_response_schema(self, schema):
        """
        接口文档返回自定义结构
        使用 drf_spectacular 自动生成接口文档，有效 type 类型如下
        from drf_spectacular.utils import OpenApiTypes
        type: string | integer | number | boolean | object | array

        Args:
            schema:

        Returns:

        """
        return {
            'type': 'object',
            'required': ['count', 'data'],
            'properties': {
                'status': {
                    'type': 'boolean',
                    'example': True,
                },
                'code': {
                    'type': 'integer',
                    'example': 200,
                },
                'msg': {
                    'type': 'string',
                    'example': 'Queried successfully.',
                },
                'count': {
                    'type': 'integer',
                    'example': 123,
                },
                'next': {
                    'type': 'string',
                    'nullable': True,
                    'format': 'uri',
                    'example': 'http://api.example.org/accounts/?{offset_param}=400&{limit_param}=100'.format(
                        offset_param=self.offset_query_param, limit_param=self.limit_query_param),
                },
                'previous': {
                    'type': 'string',
                    'nullable': True,
                    'format': 'uri',
                    'example': 'http://api.example.org/accounts/?{offset_param}=200&{limit_param}=100'.format(
                        offset_param=self.offset_query_param, limit_param=self.limit_query_param),
                },
                'data': schema,
            },
        }


class CustomBasePageNumberPagination(PageNumberPagination):
    """自定义页码分页器基类"""

    # 默认显示每页的数据条数限制
    page_size = 10
    # 每页最大显示数据条数
    max_page_size = 200
    # 页码请求传参参数字段名
    page_query_param = 'number'
    # 每页数据限制请求传参参数字段名
    page_size_query_param = 'size'

    def get_paginated_response(self, data):
        status = True
        code = 200
        msg = 'Queried successfully.'
        if not data:
            # status = False  # 查不到，理论来说，只是结果为空，但不是出错。
            code = 404
            msg = "Data not found."

        return Response(OrderedDict([
            ('status', status),
            ('code', code),
            ('msg', msg),
            ('count', self.page.paginator.count),
            ('number', self.page.number),
            ('size', len(data)),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('data', data)
        ]))


class RbacPageNumberPagination(CustomBasePageNumberPagination):
    # 每页最大显示数据条数
    max_page_size = 9999

    def get_paginated_response(self, data):
        status = True
        code = 200
        msg = 'Queried successfully.'
        if not data:
            # status = False  # 查不到，理论来说，只是结果为空，但不是出错。
            code = 404
            msg = "Data not found."

        data_info = {'data': OrderedDict([
            ('status', status),
            ('code', code),
            ('msg', msg),
            ('count', self.page.paginator.count),
            ('number', self.page.number),
            ('size', len(data)),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('content', data)
        ])}
        return Response(data_info, content_type="application/json; charset=utf-8")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-11-15 11:39
# @Author  : Jieay
# @File    : health_view.py

from utils.frame.api_view_external import BaseOpenApiView, BaseResponse
from utils.frame.drf_spectacular_external import CustomDrfSpectacularResponse
from drf_spectacular.utils import extend_schema
import logging

logger = logging.getLogger(__name__)


class GetHealthCheckView(BaseOpenApiView):
    """健康检查信息"""

    @extend_schema(
        tags=['Common'],
        summary='HealthCheck',
        description='健康检查接口',
        methods=['GET'],
        responses=CustomDrfSpectacularResponse().api_response(
            data={
                200: CustomDrfSpectacularResponse().success_response(
                    code=200,
                    data={'check': 'ok'}
                ),
                400: CustomDrfSpectacularResponse().failed_response(code=400)
            }
        )
    )
    def get(self, request):
        data = {'check': 'ok'}
        return BaseResponse(code=200, data=data, status=True, msg='Queried successfully.')
